/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.feeds;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Base64;

/**
 * Class representing a feed favicon that supports serialization
 * @author lillicoder
 */
public class Favicon implements Serializable {

	private static final String EXCEPTION_NULL_CONSTRUCTOR_BITMAP = 
		"The given Bitmap must not be null.";

	/**
	 * Serialization ID.
	 */
	private static final long serialVersionUID = 2250679806500414860L;

	private Bitmap _image;
	
	public Favicon(Bitmap image) {
		if (image == null)
			throw new IllegalArgumentException(EXCEPTION_NULL_CONSTRUCTOR_BITMAP);
		
		this.setImage(image);
	}
	
	/**
	 * Gets the {@link Bitmap} image for this favicon.
	 * @return Favicon {@link Bitmap} image.
	 */
	public Bitmap getImage() {
		return this._image;
	}
	
	/**
	 * Sets the {@link Bitmap} image for this favicon.
	 * @param image {@link Bitmap} image to set.
	 */
	private void setImage(Bitmap image) {
		this._image = image;
	}
	
	/*
	 * Serialization methods.
	 */
	private void writeObject(ObjectOutputStream out) throws IOException {
		ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
		
		Bitmap image = this.getImage();
		image.compress(Bitmap.CompressFormat.PNG, 100, byteOutput);
		
		// To avoid having to manually handle input bytes in readObject(),
		// just pass a base64 encoded string.
		byte[] rawImage = byteOutput.toByteArray();
		String encodedImage = Base64.encodeToString(rawImage, Base64.DEFAULT);
		out.writeObject(encodedImage);
	}
	
	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		// Incoming data is a base64 image, decode and set the image.
		String encodedImage = (String) in.readObject();
		byte[] rawImage = Base64.decode(encodedImage, Base64.DEFAULT);
		Bitmap image = BitmapFactory.decodeByteArray(rawImage, 0, rawImage.length);
		
		this.setImage(image);
	}
}