/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.feeds;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.Assert;
import android.util.Log;

/**
 * Lightweight, in-memory cache of {@link Feed}.
 * @author lillicoder
 */
public class FeedCache {

	private static final String TAG = FeedCache.class.getSimpleName();

	private static final String WARNING_NO_FEED_FOR_GIVEN_ID = 
		"There is no feed in the repository with an ID of %d, skipping ID";
	
	private static Map<Integer, Feed> sFeedsById;
	
	public FeedCache() {
		if (sFeedsById == null)
			this.setFeedsById(new HashMap<Integer, Feed>());
	}
	
	/**
	 * Gets the in-memory map of feeds keyed by ID. 
	 * @return {@link Map} of {@link Feed} keyed by ID.
	 */
	private synchronized Map<Integer, Feed> getFeedsById() {
		return sFeedsById;
	}

	/**
	 * Gets the {@link Feed} those ID matches the given feed ID.
	 * If no match is found, <code>null</code> is returned.
	 * @param feedId Feed ID of the feed to get.
	 * @return {@link Feed} whose ID matches the given feed ID,
	 * 		   <code>null</code> if no such feed exists.
	 */
	public Feed getFeed(int feedId) {
		Map<Integer, Feed> feedsById = this.getFeedsById();
		return feedsById.get(feedId);
	}
	
	/**
	 * Gets a collection of {@link Feed} containing all feeds
	 * matching the given array of feed IDs. If there is no
	 * corresponding feed for a given feed ID, that ID is ignored.
	 * @param feedIds IDs of the feeds to get.
	 * @return {@link List} of {@link Feed} matching the set of given IDs.
	 */
	public List<Feed> getFeeds(int[] feedIds) {
		List<Feed> feeds = new ArrayList<Feed>();
		
		Map<Integer, Feed> feedsById = this.getFeedsById();
		for (int feedId : feedIds) {
			Feed feed = feedsById.get(feedId);
			if (feed == null)
				Log.w(TAG, String.format(WARNING_NO_FEED_FOR_GIVEN_ID, feedId));
			else
				feeds.add(feed);
		}
		
		return feeds;
	}

	/**
	 * Sets the in-memory map of feeds keyed by feed ID.
	 * This is intended to be used with {@link #restoreFeeds()}
	 * when restoring persisted feeds in repository instances.
	 */
	private synchronized void setFeedsById(Map<Integer, Feed> feedsById) {
		sFeedsById = feedsById;
	}
	
	/**
	 * Updates the repository of feeds to include the feeds in the given
	 * map. Existing feeds in the repository with a matching feed ID
	 * will be replaced by the feed in the given map.
	 * @param feedsById {@link Map} of {@link Feed} keyed by ID.
	 */
	public void updateFeeds(Map<Integer, Feed> feedsById) {
		Assert.assertTrue(feedsById != null);
		
		Map<Integer, Feed> repositoryFeeds = this.getFeedsById();
		repositoryFeeds.putAll(feedsById);
	}
		
}