/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.feeds;

import java.util.ArrayList;
import java.util.List;

/**
 * Model for a collection of {@link Feed}. Folders can be nested
 * inside of other folders.
 * @author lillicoder
 */
public class FeedFolder implements IFeed {

	/**
	 * Serialization ID.
	 */
	private static final long serialVersionUID = -9017918954727665426L;
	
	private FeedFolder _parent;
	private List<FeedFolder> _childFolders;
	private List<Feed> _childFeeds;
	
	private String _name;
	private UnreadCounts _unreadCounts;
	
	/**
	 * Instantiates a new folder instance with the given name and no parent folder.
	 */
	public FeedFolder(String name) {
		this(name, null);
	}
	
	/**
	 * Instantiates a new folder instance with the given name and parent {@link FeedFolder}.
	 * @param parent Parent {@link FeedFolder} for this instance.
	 */
	public FeedFolder(String name, FeedFolder parent) {
		this.setName(name);
		this.setParent(parent);
		this.setChildFeeds(new ArrayList<Feed>());
		this.setChildFolders(new ArrayList<FeedFolder>());
		this.setUnreadCounts(new UnreadCounts());
	}

	/**
	 * Adds the given {@link Feed} to this folder's collection
	 * of child feeds. 
	 * @param feedId {@link Feed} to add to this folder.
	 */
	public void addChildFeed(Feed feed) {
		List<Feed> childFeeds = this.getChildFeeds();
		childFeeds.add(feed);
		
		this.updateUnreadCounts(feed);
	}
	
	/**
	 * Adds the given {@link FeedFolder} to this folder's
	 * collection of child folders.
	 * @param folder {@link FeedFolder} to add to this folder.
	 */
	public void addChildFolder(FeedFolder folder) {
		List<FeedFolder> childFolders = this.getChildFolders();
		childFolders.add(folder);
		
		this.updateUnreadCounts(folder);
	}

	@Override
	public int compareTo(IFeed another) {
		if (another == null)
			return 1;
		
		// Folders come after any child-less feeds. 
		if (! another.hasChildren())
			return -1;
		
		// Sort by name (alphabetical).
		String name = this.getName();
		String anotherName = another.getName();
		return name.compareTo(anotherName);
	}

	@Override
	public List<IFeed> getChildren() {
		List<IFeed> children = new ArrayList<IFeed>();
		
		List<FeedFolder> childFolders = this.getChildFolders();
		if (childFolders != null)
			children.addAll(childFolders);
		
		List<Feed> childFeeds = this.getChildFeeds();
		if (childFeeds != null)
			children.addAll(childFeeds);
		
		return children;
	}
	
	@Override
	public boolean hasChildren() {
		List<IFeed> children = this.getChildren();
		return children != null && ! children.isEmpty();
	}
	
	/**
	 * Gets the collection of child {@link Feed} for this folder.
	 * @return {@link List} of child {@link Feed}.
	 */
	private List<Feed> getChildFeeds() {
		return this._childFeeds;
	}
	
	/**
	 * Gets the collection of child {@link FeedFolder} for this folder.
	 * @return {@link List} of child {@link FeedFolder}.
	 */
	private List<FeedFolder> getChildFolders() {
		return this._childFolders;
	}
	
	@Override
	public Favicon getFavicon() {
		// By convention, feed folders do not have favicons.
		return null;
	}
	
	@Override
	public String getName() {
		return this._name;
	}
	
	/**
	 * Gets this folder's parent {@link FeedFolder}.
	 * @return Parent {@link FeedFolder}, 
	 * 		   <code>null</code> if this folder has no parent.
	 */
	public FeedFolder getParent() {
		return this._parent;
	}

	@Override
	public UnreadCounts getUnreadCounts() {
		return this._unreadCounts;
	}

	/**
	 * Sets this folder's collection of child {@link Feed}
	 * to the given collection. This is intended for instance
	 * construction.
	 * @param childFeeds {@link List} of child {@link Feed} to set.
	 */
	private void setChildFeeds(List<Feed> childFeeds) {
		this._childFeeds = childFeeds;
	}
	
	/**
	 * Sets this folder's collection of child feed IDs
	 * to the given collection. This is inteded for instance
	 * construction.
	 * @param childFolders {@link List} of child {@link FeedFolder} to set.
	 */
	private void setChildFolders(List<FeedFolder> childFolders) {
		this._childFolders = childFolders;
	}
	
	/**
	 * Sets this folder's parent {@link FeedFolder}. Parent folders
	 * are optional and this value may be set to <code>null</code>.
	 */
	private void setParent(FeedFolder parent) {
		this._parent = parent;
	}
	
	/**
	 * Sets this folder's name.
	 * @param name Folder name to set.
	 */
	private void setName(String name) {
		this._name = name;
	}
	
	/**
	 * Sets the {@link UnreadCounts} for this folder.
	 * @param unreadCounts Folder {@link UnreadCounts}.
	 */
	private void setUnreadCounts(UnreadCounts unreadCounts) {
		this._unreadCounts = unreadCounts;
	}
	
	/**
	 * Updates the unread counts for this folder with the
	 * unread counts from the given {@link IFeed}.
	 * @param feed {@link IFeed} whose unread counts will be added.
	 */
	private void updateUnreadCounts(IFeed feed) {
		UnreadCounts counts = this.getUnreadCounts();
		UnreadCounts newCounts = feed.getUnreadCounts();
		counts.addCounts(newCounts);
	}

}