/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.feeds;

import junit.framework.Assert;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import com.lillicoder.newsblurry.R;

/**
 * List item view for the feeds list.
 * @author lillicoder
 */
public class FeedListItemView extends FrameLayout {

	private static final int FOLDER_FAVICON_RESOURCE = R.drawable.ic_folder_favicon;
	
	private ImageView _favicon;
	private TextView _title;
	
	private TextView _unreadLikes;
	private TextView _unreadNeutrals;
	private TextView _unreadDislikes;
	
	public FeedListItemView(Context context) {
		this(context, null);
	}
	
	public FeedListItemView(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}
	
	public FeedListItemView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		
		LayoutInflater inflater = LayoutInflater.from(context);
		inflater.inflate(R.layout.list_item_feed, this);
		
		this.initializeReferences();
	}
	
	/**
	 * Sets the {@link IFeed} this view should display.
	 * @param feed {@link IFeed} to display.
	 */
	public void setFeed(IFeed feed) {
		Assert.assertTrue(feed != null);
		
		this.setTitle(feed.getName());
		
		Favicon favicon = feed.getFavicon();
		if (favicon != null)
			this.setFavicon(favicon);
		else if (feed.hasChildren())
			this.setFaviconResource(FOLDER_FAVICON_RESOURCE);
		
		UnreadCounts counts = feed.getUnreadCounts();
		this.displayUnreadCounts(counts);
	}
	
	/**
	 * Displays labels for unread count values in the given {@link UnreadCounts}.
	 * If there is no count for a {@link IntelligenceClassifier} category, no 
	 * label will be shown for that count.
	 * @param unreadCounts {@link UnreadCounts} to display.
	 */
	private void displayUnreadCounts(UnreadCounts unreadCounts) {
		int unreadLikes = unreadCounts.getLikeCount();
		int unreadNeutrals = unreadCounts.getNeutralCount();
		int unreadDislikes = unreadCounts.getDislikeCount();
		
		if (unreadLikes > 0) {
			this._unreadLikes.setText(Integer.toString(unreadLikes));
			this._unreadLikes.setVisibility(View.VISIBLE);
		} else {
			this._unreadLikes.setVisibility(View.GONE);
		}
		
		if (unreadNeutrals > 0) {
			this._unreadNeutrals.setText(Integer.toString(unreadNeutrals));
			this._unreadNeutrals.setVisibility(View.VISIBLE);
		} else {
			this._unreadNeutrals.setVisibility(View.GONE);
		}
		
		if (unreadDislikes > 0) {
			this._unreadDislikes.setText(Integer.toString(unreadDislikes));
			this._unreadDislikes.setVisibility(View.VISIBLE);
		} else {
			this._unreadDislikes.setVisibility(View.GONE);
		}
	}
	
	/**
	 * Initializes child references for this view.
	 */
	private void initializeReferences() {
		this._favicon = 
			(ImageView) this.findViewById(R.id.FeedListItem_favicon);
		this._title = 
			(TextView) this.findViewById(R.id.FeedListItem_title);
		this._unreadLikes = 
			(TextView) this.findViewById(R.id.FeedListItem_unreadLikes);
		this._unreadNeutrals = 
			(TextView) this.findViewById(R.id.FeedListItem_unreadNeutrals);
		this._unreadDislikes = 
			(TextView) this.findViewById(R.id.FeedListItem_unreadDislikes);
	}
	
	/**
	 * Sets the favicon for this view. 
	 * @param favicon {@link Favicon} to set.
	 */
	private void setFavicon(Favicon favicon) {
		if (favicon != null) {
			Resources resources = this.getResources();
			Drawable faviconDrawable = new BitmapDrawable(resources, favicon.getImage());
			this._favicon.setImageDrawable(faviconDrawable);
		}
	}
	
	/**
	 * Sets the favicon for this view to the {@link Drawable} with the given resource ID.
	 * @param resourceId Resource ID of the favicon {@link Drawable} to set.
	 */
	private void setFaviconResource(int resourceId) {
		this._favicon.setImageResource(resourceId);
	}
	
	/**
	 * Sets the title label of this view.
	 * @param title Title {@link CharSequence} to set.
	 */
	private void setTitle(CharSequence title) {
		this._title.setText(title);
	}
	
}