/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.feeds;

import java.util.List;

import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;

/**
 * Adapter for use with {@link FeedsListFragment}.
 * @author lillicoder
 */
public class FeedsListAdapter extends BaseAdapter {

	private static final String EXCEPTION_NULL_FEED_DATA = 
		"Backing data at position %d is null, cannot get view at this position.";
	
	private List<IFeed> _feeds;
	
	/**
	 * Instantiates a new adapter backed by the given collection of {@link IFeed}.
	 * @param feeds Backing {@Link List} of {@link IFeed}.
	 */
	public FeedsListAdapter(List<IFeed> feeds) {
		this.setFeedsList(feeds); 
	}
	
	@Override
	public int getCount() {
		List<IFeed> feeds = this.getFeedsList();
		
		if (feeds != null)
			return feeds.size();
		else
			return 0;
	}

	@Override
	public IFeed getItem(int position) {
		List<IFeed> feeds = this.getFeedsList();
		
		if (feeds != null && feeds.size() > position)
			return feeds.get(position);
		else
			return null;
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		if (convertView == null) {
			Context context = parent.getContext();
			convertView = new FeedListItemView(context);
		}
		
		IFeed feed = this.getItem(position);
		if (feed == null)
			throw new IllegalStateException(String.format(EXCEPTION_NULL_FEED_DATA, position));

		FeedListItemView itemView = (FeedListItemView) convertView;
		itemView.setFeed(feed);
		
		return convertView;
	}

	/**
	 * Gets this adapter's backing collection of {@link IFeed}.
	 * @return Backing {@link List} of {@link IFeed}.
	 */
	private List<IFeed> getFeedsList() {
		return this._feeds;
	}
	
	/**
	 * Sets the collection of {@link IFeed} for this adapter.
	 * This is intended for instance construction, to update
	 * the adapter see {@link #updateFeeds(List)}.
	 * @param feeds {@link List} of {@link Feed} to set.
	 */
	private void setFeedsList(List<IFeed> feeds) {
		this._feeds = feeds;
	}

	/**
	 * Updates this adapter with the given collection of {@link IFeed}.
	 * @param feeds {@link List} of updated {@link IFeed}.
	 */
	public void update(List<IFeed> feeds) {
		this.setFeedsList(feeds);
		this.notifyDataSetChanged();
	}
	
}
