/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.feeds;

import java.io.Serializable;

import junit.framework.Assert;

/**
 * Wrapper class that holds the unread counts for stories
 * matching {@link IntelligenceClassifier} values. 
 * @author lillicoder
 */
public class UnreadCounts implements Serializable {

	/**
	 * Serialization ID.
	 */
	private static final long serialVersionUID = -3347200502694584324L;
	
	private int _dislikeCount;
	private int _neutralCount;
	private int _likeCount;
	
	/**
	 * Instantiates this instance with no unread counts.
	 */
	public UnreadCounts() {
		this(0, 0, 0);
	}
	
	/**
	 * Instantiates this instance with the given dislike, neutral, and like unread counts.
	 * @param dislikeCount Dislike-classified story count.
	 * @param neutralCount Neutral-classified story count.
	 * @param likeCount Like-classified story count.
	 */
	public UnreadCounts(int dislikeCount, int neutralCount, int likeCount) {
		this.setDislikeCount(dislikeCount);
		this.setNeutralCount(neutralCount);
		this.setLikeCount(likeCount);
	}
	
	/**
	 * Adds all unread counts from the given {@link UnreadCounts} to these unread counts.
	 * @param counts {@link UnreadCounts} to add.
	 */
	public void addCounts(UnreadCounts counts) {
		Assert.assertTrue(counts != null);
		
		int newLikeCount = 
			this.getLikeCount() + counts.getLikeCount();
		this.setLikeCount(newLikeCount);
		
		int newNeutralCount = 
			this.getNeutralCount() + counts.getNeutralCount();
		this.setNeutralCount(newNeutralCount);
		
		int newDislikeCount = 
			this.getDislikeCount() + counts.getDislikeCount();
		this.setDislikeCount(newDislikeCount);
	}
	
	/**
	 * Gets the count of {@link IntelligenceClassifier#DISLIKE} stories.
	 * @return Dislike-classified story count.
	 */
	public int getDislikeCount() {
		return this._dislikeCount;
	}
	
	/**
	 * Gets the count of {@link IntelligenceClassifier#LIKE} stories.
	 * @return Like-classified story count.
	 */
	public int getLikeCount() {
		return this._likeCount;
	}
	
	/**
	 * Gets the count of {@link IntelligenceClassifier#NEUTRAL} stories.
	 * @return Neutral-classified story count.
	 */
	public int getNeutralCount() {
		return this._neutralCount;
	}
	
	/**
	 * Sets the count of {@link IntelligenceClassifier#DISLIKE} stories.
	 * @param dislikeCount Count of dislike-classified stories.
	 */
	private void setDislikeCount(int dislikeCount) {
		this._dislikeCount = dislikeCount;
	}

	/**
	 * Sets the count of {@link IntelligenceClassifier#LIKE} stories.
	 * @param likeCount Count of like-classified stories.
	 */
	private void setLikeCount(int likeCount) {
		this._likeCount = likeCount;
	}
	
	/**
	 * Sets the count of {@link IntelligenceClassifier#NEUTRAL} stories.
	 * @param neutralCount Count of neutral-classified stories.
	 */
	private void setNeutralCount(int neutralCount) {
		this._neutralCount = neutralCount;
	}
	
}