/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.login;

import junit.framework.Assert;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;

import com.lillicoder.newsblurry.R;
import com.lillicoder.newsblurry.exception.ApiRequestException;
import com.lillicoder.newsblurry.exception.InvalidUserCredentialsException;
import com.lillicoder.newsblurry.feeds.FeedsActivity;
import com.lillicoder.newsblurry.net.OnTaskCompletedListener;
import com.lillicoder.newsblurry.views.LabeledProgressBar;

/**
 * Fragment for logging into NewsBlur.
 * @author lillicoder
 */
public class LoginFragment extends Fragment {

	private OnTaskCompletedListener<LoginResponse> _loginTaskListener = new OnTaskCompletedListener<LoginResponse>() {
		@Override
		public void onPreExecute() {
			LoginFragment.this.showSigningIn();
		}

		@Override
		public void onFailure(Exception exception) {
			if (exception instanceof InvalidUserCredentialsException) {
				// User credentials entered are invalid, show messaging.
				String errorMessage = 
					LoginFragment.this.getString(R.string.status_invalidUserCredentials);
				LoginFragment.this.showError(errorMessage);
			} else if (exception instanceof ApiRequestException) {
				// API request failed but had valid credentials,
				// inform user to check connectivity.
				String errorMessage = 
					LoginFragment.this.getString(R.string.status_connectionFailure);
				LoginFragment.this.showError(errorMessage);
			} else {
				// Unable to determine error case, show generic message.
				String errorMessage = 
					LoginFragment.this.getString(R.string.status_unableToLogIn);
				LoginFragment.this.showError(errorMessage);
			}
		}

		@Override
		public void onSuccess(LoginResponse result) {
			if (result.isAuthenticated()) {
				Intent feeds = new Intent(LoginFragment.this.getActivity(), FeedsActivity.class);
				LoginFragment.this.startActivity(feeds);
				
				// We do not want the user to come back here, so finish.
				LoginFragment.this.getActivity().finish();
			} else {
				if (result.getReponseCode() < 0) {
					// Response indicates invalid credentials given,
					// show invalid credentials error.
					String errorMessage = 
						LoginFragment.this.getString(R.string.status_invalidUserCredentials);
					LoginFragment.this.showError(errorMessage);
				} else {
					// Unable to determine error case, show generic message.
					String errorMessage = 
						LoginFragment.this.getString(R.string.status_unableToLogIn);
					LoginFragment.this.showError(errorMessage);
				}
			}
		}
	};
	
	private Button _signInButton;
	private EditText _userNameInput;
	private EditText _passwordInput;
	private LabeledProgressBar _labeledProgressBar;
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View root = inflater.inflate(R.layout.fragment_login, null);
		
		this._signInButton = 
			(Button) root.findViewById(R.id.LoginFragment_signInButton);
		this._userNameInput = 
			(EditText) root.findViewById(R.id.LoginFragment_userNameInput);
		this._passwordInput = 
			(EditText) root.findViewById(R.id.LoginFragment_passwordInput);
		this._labeledProgressBar = 
			(LabeledProgressBar) root.findViewById(R.id.LoginFragment_labeledProgressBar);
		
		return root;
	}
	
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		this._passwordInput.setOnEditorActionListener(new OnEditorActionListener() {
			@Override
			public boolean onEditorAction(TextView view, int actionId, KeyEvent event) {
				// When the 'done' key is pressed, attempt login via
				// submit button.
				if (actionId == EditorInfo.IME_ACTION_DONE) {
					LoginFragment.this._signInButton.performClick();
					return true;
				}
				
				return false;
			}
		});
		
		this._signInButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View view) {
				// Gather user credentials and attempt login.
				String userName = LoginFragment.this._userNameInput.getText().toString();
				String password = LoginFragment.this._passwordInput.getText().toString();
				
				// If the password is null (which it shouldn't be), replace it with
				// an empty string for the login task.
				if (password == null)
					password = "";
				
				if (! TextUtils.isEmpty(userName)) {
					LoginTask login = 
						new LoginTask(view.getContext(), LoginFragment.this._loginTaskListener);
					login.execute(userName, password);
				} else {
					// The user provided no user name, show user name requirement error.
					LoginFragment.this.showUserNameRequiredError();
				}
			}
		});
	}

	/**
	 * Sets a status label to the given error message and adjusts
	 * the status label text color to red. The progress bar next
	 * to the status label will be hidden.
	 * @param errorMessage Error message to display.
	 */
	private void showError(String errorMessage) {
		Assert.assertTrue(! TextUtils.isEmpty(errorMessage));
		
		// We can just repurpose the labeled progress bar here
		// by setting a new message, text color, and hiding the progress bar.
		this._labeledProgressBar.setShowProgressBar(false);
		this._labeledProgressBar.setText(errorMessage);
		this._labeledProgressBar.setTextColor(this.getResources().getColor(R.color.offRed));
		this._labeledProgressBar.setVisibility(View.VISIBLE);
	}
	
	/**
	 * Convenience method that show the logging in status
	 * label and progress bar.
	 */
	private void showSigningIn() {
		this._labeledProgressBar.setShowProgressBar(true);
		this._labeledProgressBar.setText(this.getString(R.string.status_signingIn));
		this._labeledProgressBar.setTextColor(this.getResources().getColor(R.color.black));
		this._labeledProgressBar.setVisibility(View.VISIBLE);
	}
	
	/**
	 * Convenience method that shows the user name required
	 * error status label.
	 */
	private void showUserNameRequiredError() {
		String errorMessage = this.getString(R.string.status_userNameRequired);
		this.showError(errorMessage);
	}
	
}