/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.login;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.util.Log;

import com.lillicoder.newsblurry.R;
import com.lillicoder.newsblurry.exception.ApiRequestException;
import com.lillicoder.newsblurry.exception.InvalidUserCredentialsException;
import com.lillicoder.newsblurry.net.BaseTask;
import com.lillicoder.newsblurry.net.OnTaskCompletedListener;

/**
 * Task that logs in the user indicated by the given user name and password parameters.
 * @author lillicoder
 */
public class LoginTask extends BaseTask<String, Void, LoginResponse> {

	private static final String TAG = LoginTask.class.getSimpleName();

	private static final String EXCEPTION_UNABLE_TO_GET_USER_NAME_AND_PASSWORD_PARAMS = 
		"Unable to get both user name and password parameters, were both passed in to execute()?";
	
	public LoginTask(Context context, OnTaskCompletedListener<LoginResponse> listener) {
		super(context, listener);
	}

	@Override
	protected LoginResponse doInBackground(String... params) {
		LoginResponse response = null;
		
		try {
			// Create the HttpPost instance for this login request and attach the login parameters.
			Context context = this.getContext();
			
			String loginUrl = context.getString(R.string.api_loginUrl);
			HttpPost loginPost = new HttpPost(loginUrl);
			
			List<NameValuePair> loginParameters = this.getLoginParameters(params[0], params[1]);
			loginPost.setEntity(new UrlEncodedFormEntity(loginParameters));
			
			// Pull entity and parse JSON response.
			HttpResponse httpResponse = this.executeWebRequest(loginPost);
			JSONObject responseJson = this.getJsonFromResponse(httpResponse);
			
			LoginResponseParser parser = new LoginResponseParser(); 
			response = parser.parseLoginResponse(responseJson);
		} catch (IndexOutOfBoundsException e) {
			Log.e(TAG, EXCEPTION_UNABLE_TO_GET_USER_NAME_AND_PASSWORD_PARAMS, e);
			this.setException(new InvalidUserCredentialsException(e));
		} catch (ApiRequestException e) {
			Log.e(TAG, EXCEPTION_API_REQUEST_FAILED, e);
			this.setException(e);
		} catch (IOException e) {
			Log.e(TAG, EXCEPTION_FAILED_TO_PARSE_RESPONSE, e);
			this.setException(e);
		} catch (JSONException e) {
			Log.e(TAG, EXCEPTION_UNABLE_TO_CREATE_JSON_FROM_RESPONSE, e);
			this.setException(e);
		}
		
		return response;
	}
	
	/**
	 * Gets a collection of parameters for use with a login POST request.
	 * @param userName User name parameter to add to the collection.
	 * @param password Password paramter to add to the collection.
	 * @return {@link List} of {@link NameValuePair} parameters for use with a login POST request.
	 */
	private List<NameValuePair> getLoginParameters(String userName, String password) {
		List<NameValuePair> loginParameters = new ArrayList<NameValuePair>(2);
		loginParameters.add(new BasicNameValuePair("username", userName));
		loginParameters.add(new BasicNameValuePair("password", password));
        
		return loginParameters;
	}
	
}