/**
 * Copyright 2012 Scott Weeden-Moody
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.lillicoder.newsblurry.views;

import android.content.Context;
import android.content.res.Resources;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.lillicoder.newsblurry.R;

/**
 * Custom progress bar view that has both a {@link ProgressBar} and
 * a {@link TextView} for descriptive loading text.
 * @author lillicoder
 */
public class LabeledProgressBar extends RelativeLayout {

	private ProgressBar _progressBar;
	private TextView _label;
	
	public LabeledProgressBar(Context context) {
		this(context, null);
	}
	
	public LabeledProgressBar(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}
	
	public LabeledProgressBar(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		
		LayoutInflater inflater = LayoutInflater.from(context);
		inflater.inflate(R.layout.view_labeled_progress_bar, this);
		
		this.initializeReferences();
	}
	
	/**
	 * Initializes child references for this view.
	 */
	private void initializeReferences() {
		this._progressBar = 
			(ProgressBar) this.findViewById(R.id.LabeledProgressBar_progressBar);
		this._label = 
			(TextView) this.findViewById(R.id.LabeledProgressBar_label);
	}
	
	/**
	 * Shows or hides this view's label. If set to hide,
	 * space used by the label will be freed.
	 * @param showProgressBar <code>true</code> to show progress bar,
	 * 						  <code>false</code> to hide progress bar.
	 */
	public void setShowLabel(boolean showLabel) {
		if (showLabel)
			this._label.setVisibility(View.VISIBLE);
		else
			this._label.setVisibility(View.GONE);
	}
	
	/**
	 * Shows or hides this view's {@link ProgressBar}. If set to hide,
	 * space used by the progress bar will be freed.
	 * @param showProgressBar <code>true</code> to show progress bar,
	 * 						  <code>false</code> to hide progress bar.
	 */
	public void setShowProgressBar(boolean showProgressBar) {
		if (showProgressBar)
			this._progressBar.setVisibility(View.VISIBLE);
		else
			this._progressBar.setVisibility(View.GONE);
	}
	
	/**
	 * Sets the text for this view.
	 * @param label {@link CharSequence} to set as label.
	 */
	public void setText(CharSequence label) {
		this._label.setText(label);
	}
	
	/**
	 * Sets the text color for this view. The given color
	 * value must be a resource color value, see {@link Resources#getColor(int)}.
	 * @param color Text color to set.
	 */
	public void setTextColor(int color) {
		this._label.setTextColor(color);
	}

	/**
	 * Sets this view to show the given message as an error.
	 * If the given message is <code>null</code> or empty, no
	 * message will be displayed.
	 * @param message Message to display as an error.
	 */
	public void showErrorMessage(String message) {
		// We do not need special handling for a null message
		// since the label TextView will disappear when set with an
		// empty string.
		Resources resources = this.getResources();
		int textColor = resources.getColor(R.color.offRed);
		
		this.setShowLabel(true);
		this.setShowProgressBar(false);
		this.setText(message);
		this.setTextColor(textColor);
		this.setVisibility(View.VISIBLE);
	}
	
	/**
	 * Sets this view to show its loading state with no message.
	 */
	public void showLoading() {
		this.showLoading(null);
	}
	
	/**
	 * Sets this view to show its loading state with the given message.
	 * If the given message is <code>null</code> or empty, no message
	 * will be displayed. 
	 * @param message Optional massage to display.
	 */
	public void showLoading(String message) {
		// We do not need special handling for a null message
		// since the label TextView will disappear when set with an
		// empty string.
		Resources resources = this.getResources();
		int textColor = resources.getColor(R.color.black);
		
		this.setShowLabel(true);
		this.setText(message);
		this.setTextColor(textColor);
		this.setShowProgressBar(true);
		this.setVisibility(View.VISIBLE);
	}
	
}